package com.zswl.cloud.springBtach.server.core.api.xiaoju.sdk.util;

import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.Proxy;
import java.net.URL;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPInputStream;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

public abstract class HttpClient {
    private static int connectTimeout = 15000;

    private static int readTimeout = 30000;

    private static final String DEFAULT_CHARSET = "UTF-8";

    private static boolean ignoreSSLCheck = true;

    private static boolean ignoreHostCheck = true;

    public static class TrustAllTrustManager implements X509TrustManager {
        public X509Certificate[] getAcceptedIssuers() {
            return null;
        }

        public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        }

        public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        }
    }

    public static void setIgnoreSSLCheck(boolean ignoreSSLCheck) {
        HttpClient.ignoreSSLCheck = ignoreSSLCheck;
    }

    public static void setIgnoreHostCheck(boolean ignoreHostCheck) {
        HttpClient.ignoreHostCheck = ignoreHostCheck;
    }

    public static HttpResponseData doPost(String url, byte[] content, Map<String, String> headerMap) throws IOException {
        return doPost(url, "application/json;charset=utf-8", content, connectTimeout, readTimeout, headerMap, null);
    }

    public static HttpResponseData doPost(String url, byte[] content, Map<String, String> headerMap, Proxy proxy) throws IOException {
        return doPost(url, "application/json;charset=utf-8", content, connectTimeout, readTimeout, headerMap, proxy);
    }

    public static HttpResponseData doPost(String url, byte[] content, int connectTimeout, int readTimeout, Map<String, String> headerMap) throws IOException {
        return doPost(url, "application/json;charset=utf-8", content, connectTimeout, readTimeout, headerMap, null);
    }

    private static HttpResponseData doPost(String url, String ctype, byte[] content, int connectTimeout, int readTimeout, Map<String, String> headerMap, Proxy proxy) throws IOException {
        HttpURLConnection conn = null;
        OutputStream out = null;
        String rsp = null;
        HttpResponseData data = new HttpResponseData();
        try {
            conn = getConnection(new URL(url), "POST", ctype, headerMap, proxy);
            conn.setConnectTimeout(connectTimeout);
            conn.setReadTimeout(readTimeout);
            out = conn.getOutputStream();
            out.write(content);
            rsp = getResponseAsString(conn);
            data.setBody(rsp);
            Map<String, List<String>> headers = conn.getHeaderFields();
            data.setHeaders(headers);
        } finally {
            if (out != null)
                out.close();
            if (conn != null)
                conn.disconnect();
        }
        return data;
    }

    private static HttpURLConnection getConnection(URL url, String method, String ctype, Map<String, String> headerMap, Proxy proxy) throws IOException {
        HttpURLConnection conn = null;
        if (proxy == null) {
            conn = (HttpURLConnection) url.openConnection();
        } else {
            conn = (HttpURLConnection) url.openConnection(proxy);
        }
        if (conn instanceof HttpsURLConnection) {
            HttpsURLConnection connHttps = (HttpsURLConnection) conn;
            if (ignoreSSLCheck) {
                try {
                    SSLContext ctx = SSLContext.getInstance("TLS");
                    ctx.init(null, new TrustManager[]{new TrustAllTrustManager()}, new SecureRandom());
                    connHttps.setSSLSocketFactory(ctx.getSocketFactory());
                    connHttps.setHostnameVerifier(new HostnameVerifier() {
                        public boolean verify(String hostname, SSLSession session) {
                            return true;
                        }
                    });
                } catch (Exception e) {
                    throw new IOException(e.toString());
                }
            } else if (ignoreHostCheck) {
                connHttps.setHostnameVerifier(new HostnameVerifier() {
                    public boolean verify(String hostname, SSLSession session) {
                        return true;
                    }
                });
            }
            conn = connHttps;
        }
        conn.setRequestMethod(method);
        conn.setDoInput(true);
        conn.setDoOutput(true);
        conn.setRequestProperty("Host", url.getHost());
        conn.setRequestProperty("Content-Type", ctype);
        if (headerMap != null)
            for (Map.Entry<String, String> entry : headerMap.entrySet())
                conn.setRequestProperty(entry.getKey(), entry.getValue());
        return conn;
    }

    protected static String getResponseAsString(HttpURLConnection conn) throws IOException {
        String charset = getResponseCharset(conn.getContentType());
        if (conn.getResponseCode() < 400) {
            String contentEncoding = conn.getContentEncoding();
            if ("gzip".equalsIgnoreCase(contentEncoding))
                return getStreamAsString(new GZIPInputStream(conn.getInputStream()), charset);
            return getStreamAsString(conn.getInputStream(), charset);
        }
        if (conn.getResponseCode() == 400) {
            InputStream error = conn.getErrorStream();
            if (error != null)
                return getStreamAsString(error, charset);
        }
        throw new IOException(conn.getResponseCode() + " " + conn.getResponseMessage());
    }

    public static String getStreamAsString(InputStream stream, String charset) throws IOException {
        try {
            Reader reader = new InputStreamReader(stream, charset);
            StringBuilder response = new StringBuilder();
            char[] buff = new char[1024];
            int read = 0;
            while ((read = reader.read(buff)) > 0)
                response.append(buff, 0, read);
            return response.toString();
        } finally {
            if (stream != null)
                stream.close();
        }
    }

    public static String getResponseCharset(String ctype) {
        String charset = "UTF-8";
        if (!StringUtils.isEmpty(ctype)) {
            String[] params = ctype.split(";");
            for (String param : params) {
                param = param.trim();
                if (param.startsWith("charset")) {
                    String[] pair = param.split("=", 2);
                    if (pair.length == 2 &&
                            !StringUtils.isEmpty(pair[1]))
                        charset = pair[1].trim();
                    break;
                }
            }
        }
        return charset;
    }
}
